---------------------------------------------------------------------------------------	
--                                CONTROLE_COUNTER    
---------------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use work.packet.all;

-- interface do Controle_Counter
entity Controle_Counter is
	port
	(
		clock: 		in std_logic;
		reset: 		in std_logic;
		ack_h:  	in std_logic;
		ack_tx:  	in std_logic;
		data_av:  	in std_logic;
		free:		out std_logic;
		data_in:	in reg8);
end Controle_Counter;

-- implementao do Controle_Counter
architecture Controle_Counter of Controle_Counter is

type stateCounter is (SC0,SC1,SC2,SC3,SC4,SC5);
signal SC,PSC: stateCounter;
signal count: reg8;

begin
	process(reset,clock)
	begin
		if reset='1' then
			SC<=SC0;
		elsif clock'event and clock='1' then
			SC<=PSC;
		end if;
	end process;
	
	------------------------------------------------------------------------------------------------------	
	-- PARTE COMBINACIONAL PARA DEFINIR O PRXIMO ESTADO DO COUNTER
	------------------------------------------------------------------------------------------------------	
	process(SC,ack_h,ack_tx,data_av,count)
	begin
		case SC is
			when SC0 => if ack_h = '1' then PSC <= SC1; else PSC <= SC0; end if;
			when SC1 => if ack_tx = '1' then PSC <= SC2; else PSC <= SC1; end if;
			when SC2 => if data_av = '1' then PSC <= SC3; else PSC <= SC2; end if;
			when SC3 => if ack_tx = '1' then PSC <= SC4; else PSC <= SC3; end if;
			when SC4 => if ack_tx = '1' then PSC <= SC5; else PSC <= SC4; end if;
			when SC5 => if count = x"0" then PSC <= SC0; else PSC <= SC4; end if;
		end case;
	end process;

	------------------------------------------------------------------------------------------------------	
	-- executa as aes correspondente ao estado atual da mquina de estados COUNTER
	------------------------------------------------------------------------------------------------------	
	process(clock)
	begin
		if clock'event and clock='0' then
			case SC is
				when SC0 =>
					free <= '1';
				when SC1 =>
					free <= '0';
				when SC3 =>
					count <= data_in;
				when SC5 =>
					count <= count - 1;
				when others => null;
			end case;
		end if;
	end process;

end Controle_Counter;