library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use work.packet.all;

-- interface da Fila
entity Controle_Mux is
	port(
	clock:			in std_logic;
	reset:			in std_logic;
	address: 		in reg8;
	req_rot:		in std_logic;
	ack_rot:		out std_logic;
	dir_incoming:		in reg3;
	header:			in reg8;
	free:			in reg5;
	busy_out:		out reg5;
	mux_in:			out array5_reg3;
	mux_out:		out array5_reg3);
end Controle_Mux;

architecture Controle_Mux of Controle_Mux is

type state_controle_mux is (SC0,SC1,SC2,SC3,SC4,SC5,SC6);
signal SC,PSC: state_controle_mux;

signal dirx,diry: integer;
signal lx,ly,tx,ty: reg4;
signal free_ant, busy_target: reg5;
signal source:	array5_reg3;

begin
	busy_out <= busy_target;
	
	mux_in <= source;
	
	lx <= address(7 downto 4);	
	ly <= address(3 downto 0);	
	
	tx <= header(7 downto 4);
	ty <= header(3 downto 0);
	
	dirx <= WEST when lx > tx else EAST;
	diry <= NORTH when ly > ty else SOUTH;

	process(reset,clock)
	begin
		if reset='1' then
			SC <= SC0;
		elsif clock'event and clock='1' then
			SC <= PSC;
		end if;
	end process;

	------------------------------------------------------------------------------------------------------	
	-- PARTE COMBINACIONAL PARA DEFINIR o prximo estado da mquina.
	------------------------------------------------------------------------------------------------------	
	process(SC,req_rot,lx,ly,tx,ty,busy_target,dirx,diry)
	begin
		case SC is
			when SC0 => if req_rot='1' then PSC<= SC1; else PSC<=SC0; end if;
			when SC1 => if lx = tx and ly = ty and busy_target(LOCAL)='0' then PSC<=SC2;
						elsif lx /= tx and busy_target(dirx)='0' then PSC<=SC3;
						elsif ly /= ty and busy_target(diry)='0' then PSC<=SC4;
						else PSC<=SC1; end if;
			when SC2 => PSC<=SC5;
			when SC3 => PSC<=SC5;
			when SC4 => PSC<=SC5;
			when SC5 => PSC<=SC6;
			when SC6 => if req_rot='1' then PSC<= SC1; else PSC<=SC6; end if;
		end case;
	end process;

	------------------------------------------------------------------------------------------------------	
	-- executa as aes correspondente ao estado atual da mquina de estados controle_mux
	------------------------------------------------------------------------------------------------------	
	process(clock)
	begin
		if clock'event and clock='0' then
			case SC is
				when SC0 =>
					mux_out <= (others=>(others=>'0'));
					source <= (others=>(others=>'0'));
					ack_rot <= '0';
				when SC1|SC5 =>
					ack_rot <= '0';
				when SC2 =>
					source(CONV_INTEGER(dir_incoming)) <= CONV_VECTOR(LOCAL);
					mux_out(LOCAL) <= dir_incoming;
					ack_rot<='1';
				when SC3 =>
					source(CONV_INTEGER(dir_incoming)) <= CONV_VECTOR(dirx);
					mux_out(dirx) <= dir_incoming;
					ack_rot<='1';
				when SC4 =>
					source(CONV_INTEGER(dir_incoming)) <= CONV_VECTOR(diry);
					mux_out(diry) <= dir_incoming;
					ack_rot<='1';
				when others => null;
			end case;
		end if;
	end process;

	process(reset,clock)
	begin
		if reset='1' then
			busy_target <= (others=> '0');
			free_ant <= free;
		elsif clock'event and clock='0' then
			if free(LOCAL)/=free_ant(LOCAL) then busy_target(CONV_INTEGER(source(LOCAL))) <= NOT free(LOCAL); free_ant(LOCAL)<=free(LOCAL); end if;
			if free(EAST)/=free_ant(EAST) then busy_target(CONV_INTEGER(source(EAST))) <= NOT free(EAST); free_ant(EAST)<=free(EAST); end if;
			if free(WEST)/=free_ant(WEST) then busy_target(CONV_INTEGER(source(WEST))) <= NOT free(WEST); free_ant(WEST)<=free(WEST); end if;
			if free(NORTH)/=free_ant(NORTH) then busy_target(CONV_INTEGER(source(NORTH))) <= NOT free(NORTH); free_ant(NORTH)<=free(NORTH); end if;
			if free(SOUTH)/=free_ant(SOUTH) then busy_target(CONV_INTEGER(source(SOUTH))) <= NOT free(SOUTH); free_ant(SOUTH)<=free(SOUTH); end if;
		end if;
	end process;
	
end Controle_Mux;