--------------------------------------------------------------------------
-- package com tipos basicos
--------------------------------------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.std_logic_unsigned.all;

package packet is  
---------------------------------------------------------
-- CONSTANTS INDEPENDENTES
---------------------------------------------------------
	constant EAST : integer := 0;
	constant WEST : integer := 1;
	constant NORTH : integer := 2;
	constant SOUTH : integer := 3;
	constant LOCAL : integer := 4;

	
	constant TAM_LINHA : integer := 4;
---------------------------------------------------------
-- CONSTANTS DEPENDENTES DO NUMERO DE ROTEADORES E MEMORIAS
---------------------------------------------------------
	constant NROT: integer := 4;
	constant NMEM: integer := 3;

	constant MIN_X : integer := 0;
	constant MIN_Y : integer := 0;
	constant MAX_X : integer := 1;
	constant MAX_Y : integer := 1;
	
	constant N00 : integer := 0;
	constant N10 : integer := 1;
	constant N01 : integer := 2;
	constant N11 : integer := 3;

	constant ADDRESSN00: std_logic_vector(7 downto 0):=x"00";
	constant ADDRESSN10: std_logic_vector(7 downto 0):=x"10";
	constant ADDRESSN01: std_logic_vector(7 downto 0):=x"01";
	constant ADDRESSN11: std_logic_vector(7 downto 0):=x"11";
	
---------------------------------------------------------
-- SUBTIPOS, TIPOS E FUNCOES
---------------------------------------------------------
	subtype reg3 is std_logic_vector(2 downto 0);
	subtype reg4 is std_logic_vector(3 downto 0);
	subtype reg5 is std_logic_vector(4 downto 0);
	subtype reg8 is std_logic_vector(7 downto 0);
	subtype reg10 is std_logic_vector(9 downto 0);
	subtype reg11 is std_logic_vector(10 downto 0);
	subtype reg16 is std_logic_vector(15 downto 0);
	subtype reg32 is std_logic_vector(31 downto 0);
	subtype regNmem is std_logic_vector((NMEM-1) downto 0);

	type buff is array(0 to 7) of reg8;
	type packet is array(0 to 4) of std_logic_vector(15 downto 0);

	type array5_reg3 is array(4 downto 0) of reg3;
	type array5_reg8 is array(4 downto 0) of reg8;
	type arrayNrot_reg4 is array((NROT-1) downto 0) of reg4;
	type arrayNrot_reg8 is array((NROT-1) downto 0) of reg8;
	type arrayNrot_reg32 is array((NROT-1) downto 0) of reg32;
	type arrayNmem_reg10 is array((NMEM-1) downto 0) of reg10;
	type arrayNmem_reg16 is array((NMEM-1) downto 0) of reg16;

	function CONV_VECTOR( int: integer ) return std_logic_vector;
	function CONV_VECTOR( letra : string(1 to TAM_LINHA);  pos: integer ) return std_logic_vector;
	function CONV_HEX( int : integer ) return string;
	function CONV_HEX( int: integer ) return std_logic_vector;         
	function CONV_STRING( dado : reg8) return string; 

----------------------------------------------------------------------------------
-- FUNCOES DEPENDENTE - deve ser alterada quando o numero de roteadores mudar
----------------------------------------------------------------------------------
	function CONV_NODO(x : integer; y: integer) return integer;
	function CONV_INDEXNOTIFY(nodo: reg8) return integer;

----------------------------------------------------------------------------------
-- TIPOS E FUNCOES DO PROCESSADOR R8
----------------------------------------------------------------------------------
  -- indica o tipo de instrucao decodificada pela unidade de controle  -- 28 INSTRUES
  -- os 15 umps com flags resumem-se a trs instrues: saltoR,salto, saltoD
 type instrucao is  
    ( add,  sub,  e,   ou,   oux,  addi,  subi,  ldl,   ldh,   ld,      st,    sl0,  sl1, sr0,  sr1,
      notA, nop, halt, ldsp, rts,  pop,   push,  saltoR, salto, saltoD, jsrr,  jsr, jsrd); 
  
  type microinstrucao is record
     mpc:   std_logic_vector(1 downto 0);  -- controle do mux de entrada do registrador PC
     msp:   std_logic;                     -- mux do sp (stack-pointer)
     mad:   std_logic_vector(1 downto 0);  -- mux para origem do endereco (AD) da memoria
     mreg:  std_logic;                     -- controle do dado que entra no banco de registradores
     ms2:   std_logic;                     -- origem do segundo registrador fonte (source2 - s2)
     ma:    std_logic;                     -- mux para a selecao do operando A na entrada da ULA
     mb:    std_logic_vector(1 downto 0);  -- mux para a selecao do operando B na entrada da ULA
     wpc:   std_logic;                     -- ENABLE para escrita em PC
     wsp:   std_logic;
     wir:   std_logic;
     wab:   std_logic;
     wula:  std_logic;
     wreg:  std_logic;  
     wnz:   std_logic;
     wcv:   std_logic;
     ce,rw: std_logic;                     -- Chip enable e R_Wnegado
     ula:   instrucao;
  end record;
         
  -- registrador de 16 bits, com reset e habilitacao de escrita
  component registrador
       port( ck,rst,ce:in std_logic;
             D:in  reg16;
             Q:out reg16 );
  end component;
  
  procedure somaAB  (  signal A,B: in  reg16;   signal Cin: in STD_LOGIC;
                       signal S:   out reg16;   signal Cout, Ov:out STD_LOGIC);
                       
  function is_zero  (  signal A:  in  reg16  ) return std_logic ;
  
end packet;

package body packet is

  --
  -- converte um inteiro em um std_logic_vector(2 downto 0)
  --
	function CONV_VECTOR( int: integer ) return std_logic_vector is         
		variable bin: reg3;
	begin
		case(int) is  
			when 0 => bin := "000";
			when 1 => bin := "001";
			when 2 => bin := "010";
			when 3 => bin := "011";
			when 4 => bin := "100";
			when 5 => bin := "101";
			when 6 => bin := "110";
			when 7 => bin := "111";
			when others =>  bin := "000";
		end case;
		return bin;
	end CONV_VECTOR;

  --
  -- converte um caracter de uma dada linha em um std_logic_vector
  --
  function CONV_VECTOR( letra:string(1 to TAM_LINHA);  pos: integer ) return std_logic_vector is         
     variable bin: reg4;
   begin
      case (letra(pos)) is  
              when '0' => bin := "0000";
              when '1' => bin := "0001";
              when '2' => bin := "0010";
              when '3' => bin := "0011";
              when '4' => bin := "0100";
              when '5' => bin := "0101";
              when '6' => bin := "0110";
              when '7' => bin := "0111";
              when '8' => bin := "1000";
              when '9' => bin := "1001";
              when 'A' => bin := "1010";
              when 'B' => bin := "1011";
              when 'C' => bin := "1100";
              when 'D' => bin := "1101";
              when 'E' => bin := "1110";
              when 'F' => bin := "1111";
              when others =>  bin := "0000";
      end case;
     return bin;
  end CONV_VECTOR;

	-- converte um inteiro em um string
	function CONV_HEX( int: integer ) return string is         
		variable str: string(1 to 1);
	begin
		case(int) is  
			when 0 => str := "0";
			when 1 => str := "1";
			when 2 => str := "2";
			when 3 => str := "3";
			when 4 => str := "4";
			when 5 => str := "5";
			when 6 => str := "6";
			when 7 => str := "7";
			when 8 => str := "8";
			when 9 => str := "9";
			when 10 => str := "A";
			when 11 => str := "B";
			when 12 => str := "C";
			when 13 => str := "D";
			when 14 => str := "E";
			when 15 => str := "F";
			when others =>  str := "U";
		end case;
		return str;
	end CONV_HEX;


	-- converte um inteiro em um hex
	function CONV_HEX( int: integer ) return std_logic_vector is         
		variable hex : reg4;
	begin
		case(int) is  
			when 0 => hex := x"0";
			when 1 => hex := x"1";
			when 2 => hex := x"2";
			when 3 => hex := x"3";
			when 4 => hex := x"4";
			when 5 => hex := x"5";
			when 6 => hex := x"6";
			when 7 => hex := x"7";
			when 8 => hex := x"8";
			when 9 => hex := x"9";
			when 10 => hex := x"A";
			when 11 => hex := x"B";
			when 12 => hex := x"C";
			when 13 => hex := x"D";
			when 14 => hex := x"E";
			when 15 => hex := x"F";
			when others =>  hex := x"0";
		end case;
		return hex;
	end CONV_HEX;
	
	function CONV_STRING(dado : reg8) return string is
		variable str1,str2: string(1 to 1);
		variable str: string(1 to 2); 
	begin
		str1 := CONV_HEX(CONV_INTEGER(dado(7 downto 4)));
		str2 := CONV_HEX(CONV_INTEGER(dado(3 downto 0)));
		str := str1 & str2;
		return str;
	end CONV_STRING;

---------------------------------------------------------
-- FUNCAO DEPENDENTES DO NUMERO DE ROTEADORES
---------------------------------------------------------
	function CONV_NODO(x : integer; y: integer) return integer is
		variable int: integer; 
	begin
		case(x) is
			when 0 =>
				case(y) is
					when 0 => int := N00;
					when 1 => int := N01;
--					when 2 => int := N02;
					when others => int := N00;
				end case;
			when 1 =>
				case(y) is
					when 0 => int := N10;
					when 1 => int := N11;
--					when 2 => int := N12;
					when others => int := N00;
				end case;
--			when 2 =>
--				case(y) is
--					when 0 => int := N20;
--					when 1 => int := N21;
--					when 2 => int := N22;
--					when others => int := N00;
--				end case;
			when others => int := N00;
		end case;
		return int;
	end CONV_NODO;

	function CONV_INDEXNOTIFY(nodo: reg8) return integer is
		variable int: integer; 
	begin
		case(nodo) is
			when x"00" => int := N00;
			when x"10" => int := N10;
			when x"01" => int := N01;
			when x"11" => int := N11;
			when others => int := N00;
		end case;
		return int;
	end CONV_INDEXNOTIFY;

---------------------------------------------------------
-- FUNCAO E PROCESDURES DO PROCESSADOR R8
---------------------------------------------------------
  procedure somaAB  (  signal A,B: in  reg16;   signal Cin: in STD_LOGIC;
                       signal S:   out reg16;   signal Cout, Ov:out STD_LOGIC) is             
     variable cant, carry : STD_LOGIC;
  begin    
             for w in 0 to 15 loop
                  if w=0 then carry:=Cin;  end if;
                  S(w) <= A(w) xor B(w) xor carry;
                  cant  := carry;
                  carry := (A(w) and B(w)) or (A(w) and carry) or (B(w) and carry);
             end loop;
             Cout <= carry;
             Ov <= cant xor carry;
  end somaAB;
 
  function is_zero  (  signal A:  in  reg16  ) return std_logic is           
      begin     
             if A="0000000000000000" then return '1'; else return '0';  end if;
  end is_zero;

end packet;