---------------------------------------------------------------------------------------	
--                                ROTEADOR    
---------------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use work.packet.all;

entity Roteador is
port(
	clock: 		in std_logic;
	reset:		in std_logic;
	address: 	in reg8;
	data_in: 	in array5_reg8;
	rx:		in reg5;
	ack_rx:		out reg5;
	data_out:	out array5_reg8;
	tx:		out reg5;
	ack_tx:		in reg5);
end Roteador;

architecture Roteador of Roteador is

-- interface do Fila
component Fila is
	port(
	clock:		in std_logic;
	reset:		in std_logic;
	data_in:	in reg8;
	rx:		in std_logic;
	ack_rx:		out std_logic;
	h:		out std_logic;
	ack_h:		in std_logic;
	data_av:	out std_logic;
	data:		out reg8;
	data_ack:	in std_logic;
	free:		out std_logic);
end component;

-- interface do ArbitroParaleloRotativo
component ArbitroParaleloRotativo is
port(
	clock:		in std_logic;
	reset:		in std_logic;
	ack_rot:	in std_logic;
	h:		in reg5;
	ack_h:		out reg5;
	incoming:	out reg3;
	req_rot:	out std_logic);
end component;

-- interface do Controle_Mux
component Controle_Mux is
	port(
	clock:		in std_logic;
	reset:		in std_logic;
	address:	in reg8;
	req_rot:	in std_logic;
	ack_rot:	out std_logic;
	dir_incoming:	in reg3;
	header:		in reg8;
	free:		in reg5;
	busy_out:	out reg5;
	mux_in: 	out array5_reg3;
	mux_out:	out array5_reg3);
end component;


signal h, ack_h, data_av, free, data_ack,data_av_ant: reg5;
signal req_rot, ack_rot: std_logic;
signal incoming: reg3;
signal data: array5_reg8;
signal mux_in,mux_out: array5_reg3;
signal header: reg8;
signal busy_out: reg5;

begin

	FILAS : for i in 0 to 4 generate
		F : Fila 
		port map(
		clock => clock,
		reset => reset,
		data_in => data_in(i),
		rx => rx(i),
		ack_rx => ack_rx(i),
		h => h(i),
		ack_h => ack_h(i),
		data_av => data_av(i),
		data => data(i),
		data_ack => data_ack(i),
		free=>free(i));
	end generate FILAS;

	APR: ArbitroParaleloRotativo
	port map(
	clock => clock,
	reset => reset,
	ack_rot => ack_rot,
	h => h,
	ack_h => ack_h,
	incoming => incoming,
	req_rot => req_rot);

	C_MUX: Controle_Mux
	port map(
	clock => clock,
	reset => reset,
	address => address,
	req_rot => req_rot,
	ack_rot => ack_rot,
	dir_incoming => incoming,
	header => header,
	free => free,
	busy_out => busy_out,
	mux_in => mux_in,
	mux_out => mux_out);
	
 	header <= data(CONV_INTEGER(incoming));

----------------------------------------------------------------------------------
-- OBSERVACAO:
-- quando eh sinal de saida quem determina eh o sinal mux_out
-- quando eh sinal de entrada quem determina eh mux_in
----------------------------------------------------------------------------------	
	MUXS : for i in 0 to 4 generate   
		data_out(i) <= data(CONV_INTEGER(mux_out(i))) when busy_out(i)='1' else (others=>'0');-- when free(i)='0' else (others=>'0');
		data_ack(i) <= 	ack_tx(CONV_INTEGER(mux_in(i))) when free(i)='0' else '0'; 			
		tx(i) <= data_av(CONV_INTEGER(mux_out(i))) when busy_out(i)='1' else '0';
    	end generate MUXS;       
end Roteador;
